"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const html_parsing_rules_1 = require("./html-parsing-rules");
class HighlightEngine {
    highlight(value, theme) {
        this.theme = theme;
        return this.processHTML(value);
    }
    processHTML(value) {
        this.languageDefinition = html_parsing_rules_1.htmlParsingRules;
        this.result = '';
        this.buffer = '';
        let match;
        let count;
        let index = 0;
        while (true) {
            this.languageDefinition.terminators.lastIndex = index;
            match = this.languageDefinition.terminators.exec(value);
            if (!match) {
                break;
            }
            count = this.processLexeme(value.substring(index, match.index), match[0]);
            index = match.index + count;
        }
        this.processLexeme(value.substr(index));
        return this.result;
    }
    keywordMatch(mode, match) {
        const match_str = match[0].toLowerCase();
        return mode.keywords.hasOwnProperty(match_str) && mode.keywords[match_str];
    }
    processKeywords() {
        let keyword_match, last_index, match, result;
        if (!this.languageDefinition.keywords) {
            return this.buffer;
        }
        result = '';
        last_index = 0;
        this.languageDefinition.lexemesRe.lastIndex = 0;
        match = this.languageDefinition.lexemesRe.exec(this.buffer);
        while (match) {
            result += this.buffer.substring(last_index, match.index);
            keyword_match = this.keywordMatch(this.languageDefinition, match);
            if (keyword_match) {
                result += this.addCodePart(keyword_match[0]);
            }
            else {
                result += match[0];
            }
            last_index = this.languageDefinition.lexemesRe.lastIndex;
            match = this.languageDefinition.lexemesRe.exec(this.buffer);
        }
        return result + this.buffer.substr(last_index);
    }
    processBuffer() {
        if (this.colorFunc) {
            this.result += this.colorFunc(this.processKeywords());
            if (this.processKeywords() === '>') {
                this.colorFunc = chalk_1.default.hex(this.theme.tagContent);
            }
        }
        this.buffer = '';
    }
    startNewMode(mode, something) {
        this.result += mode.className ? this.addCodePart(mode.className) : '';
        this.languageDefinition = Object.create(mode, {
            parent: { value: this.languageDefinition }
        });
    }
    processLexeme(buffer, lexeme) {
        this.buffer += buffer;
        if (lexeme == null) {
            this.processBuffer();
            return 0;
        }
        const newMode = this.subMode(lexeme);
        if (newMode) {
            return this.processNewMode(newMode, lexeme);
        }
        const end_mode = this.endOfMode(this.languageDefinition, lexeme);
        if (end_mode) {
            return this.processEndMode(lexeme, end_mode);
        }
    }
    processEndMode(lexeme, end_mode) {
        const origin = this.languageDefinition;
        if (!(origin.returnEnd || origin.excludeEnd)) {
            this.buffer += lexeme;
        }
        this.processBuffer();
        do {
            const className = this.languageDefinition.className;
            if (className && className !== 'tag') {
                this.colorFunc = chalk_1.default.hex(this.theme['tag']);
            }
            this.languageDefinition = this.languageDefinition.parent;
        } while (this.languageDefinition !== end_mode.parent);
        if (end_mode.starts) {
            if (end_mode.endSameAsBegin) {
                end_mode.starts.endRe = end_mode.endRe;
            }
            this.startNewMode(end_mode.starts, '');
        }
        return origin.returnEnd ? 0 : lexeme.length;
    }
    processNewMode(new_mode, lexeme) {
        if (new_mode.skip) {
            this.buffer += lexeme;
        }
        else {
            if (new_mode.excludeBegin) {
                this.buffer += lexeme;
            }
            this.processBuffer();
            if (!new_mode.returnBegin && !new_mode.excludeBegin) {
                this.buffer = lexeme;
            }
        }
        this.startNewMode(new_mode, lexeme);
        return new_mode.returnBegin ? 0 : lexeme.length;
    }
    addCodePart(classname) {
        this.colorFunc = chalk_1.default.hex(this.theme[classname]);
        return '';
    }
    testRe(re, lexeme) {
        let match = re && re.exec(lexeme);
        return match && match.index === 0;
    }
    endOfMode(mode, lexeme) {
        if (this.testRe(mode.endRe, lexeme)) {
            while (mode.endsParent && mode.parent) {
                mode = mode.parent;
            }
            return mode;
        }
        if (mode.endsWithParent) {
            return this.endOfMode(mode.parent, lexeme);
        }
    }
    subMode(lexeme) {
        for (let i = 0, length = this.languageDefinition.contains.length; i < length; i++) {
            if (this.testRe(this.languageDefinition.contains[i].beginRe, lexeme)) {
                if (this.languageDefinition.contains[i].endSameAsBegin) {
                    this.languageDefinition.contains[i].endRe = this.escapeRe(this.languageDefinition.contains[i].beginRe.exec(lexeme)[0]);
                }
                return this.languageDefinition.contains[i];
            }
        }
    }
    escapeRe(value) {
        return new RegExp(value.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&'), 'm');
    }
}
exports.HighlightEngine = HighlightEngine;
//# sourceMappingURL=highlight-engine.js.map